"""
    This module provides a set of benchmark problems for global optimization.

    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math
import random
from ecspy import ec
from ecspy import emo


class Benchmark(object):
    """Defines a global optimization benchmark problem.
    
    This abstract class defines the basic structure of a global
    optimization problem. Subclasses should implement the generator 
    and evaluator methods for a particular optimization problem, 
    which can be used with ECsPy evolutionary computations. 
    
    In addition to being used with evolutionary computations, subclasses
    of this are also callable. the arguments passed to such a call are
    combined into a list and passed as the single candidate to the 
    evaluator method. The single calculated fitness is returned.
    
    Public Attributes:
    
    - *dimensions* -- the number of inputs to the problem
    - *objectives* -- the number of outputs of the problem (default 1)
    - *bounder* -- the bounding function for the problem (default None)
    - *maximize* -- whether the problem is one of maximization (default True)
    
    """
    def __init__(self, dimensions, objectives=1):
        self.dimensions = dimensions
        self.objectives = objectives
        self.bounder = None
        self.maximize = True
        
    def __str__(self):
        if self.objectives > 1:
            return '%s (%d dimensions, %d objectives)' % (self.__class__.__name__, self.dimensions, self.objectives)
        else:
            return '%s (%d dimensions)' % (self.__class__.__name__, self.dimensions)
        
    def __repr__(self):
        return '%s' % self.__class__.__name__
    
    def generator(self, random, args):
        raise NotImplementedError
        
    def evaluator(self, candidates, args):
        raise NotImplementedError
        
    def __call__(self, *args):
        candidate = [a for a in args]
        fit = self.evaluator([candidate], {})
        return fit[0]

class Binary(Benchmark):
    """Defines a binary problem based on an existing benchmark problem.
    
    This class can be used to modify an existing benchmark problem to
    allow it to use a binary representation. The generator creates
    a list of binary values of size `dimensions` x `dimension_bits`.
    The evaluator accepts a candidate represented by such a binary list
    and transforms that candidate into a real-valued list as follows:
    
    1. Each set of `dimension_bits` bits is converted to its positive
       integer representation. 
    2. Next, that integer value is divided by the maximum integer that 
       can be represented by `dimension_bits` bits to produce a real 
       number in the range [0, 1]. 
    3. That real number is then scaled to the range [lower_bound, 
       upper_bound] for that dimension (which should be defined by 
       the bounder).
    
    Public Attributes:
    
    - *benchmark* -- the original benchmark problem
    - *dimension_bits* -- the number of bits to use to represent each dimension
    - *bounder* -- the bounding function for the underlying benchmark problem
    - *maximize* -- whether the underlying benchmark problem is one of maximization
    
    """
    def __init__(self, benchmark, dimension_bits):
        Benchmark.__init__(self, benchmark.dimensions, benchmark.objectives)
        self.benchmark = benchmark
        self.dimension_bits = dimension_bits
        self.bounder = self.benchmark.bounder
        self.maximize = self.benchmark.maximize
        self.__class__.__name__ = self.__class__.__name__ + ' ' + self.benchmark.__class__.__name__
        
    def binary_to_real(self, binary):
        real = []
        for d, lo, hi in zip(range(self.dimensions), self.bounder.lower_bound, self.bounder.upper_bound):
            b = binary[d*self.dimension_bits:(d+1)*self.dimension_bits]
            real_val = float(int(''.join([str(i) for i in b]), 2))
            value = real_val / (2**(self.dimension_bits)-1) * (hi - lo) + lo
            real.append(value)
        return real
    
    def generator(self, random, args):
        return [random.choice([0, 1]) for _ in range(self.dimensions * self.dimension_bits)]
        
    def evaluator(self, candidates, args):
        binary_candidates = []
        for c in candidates:
            binary_candidates.append(self.binary_to_real(c))
        return self.benchmark.evaluator(binary_candidates, args)

        
        
#------------------------------------------------------
#             Single-objective problems
#------------------------------------------------------

class Ackley(Benchmark):
    """Defines the Ackley benchmark problem.
    
    This class defines the Ackley global optimization problem. This 
    is a multimodal minimization problem defined as follows:
    
    .. math::
    
        f(x) = -20e^{-0.2\sqrt{\\frac{1}{n} \sum_{i=1}^n x_i^2}} - e^{\\frac{1}{n} \sum_{i=1}^n \cos(2 \pi x_i)} + 20 + e
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-32, 32]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image6011.jpg
        :alt: Ackley function
        :align: center
        
        Two-dimensional Ackley function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page295.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [0, 0, ..., 0].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-32.0] * self.dimensions, [32.0] * self.dimensions)
        self.maximize = False
        self.global_optimum = [0 for _ in range(self.dimensions)]
    
    def generator(self, random, args):
        return [random.uniform(-32.0, 32.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            fitness.append(-20 * math.exp(-0.2 * math.sqrt(1.0 / self.dimensions * sum([x**2 for x in c]))) - 
                           math.exp(1.0 / self.dimensions * sum([math.cos(2 * math.pi * x) for x in c])) + 20 + math.e)
        return fitness

class Griewank(Benchmark):
    """Defines the Griewank benchmark problem.
    
    This class defines the Griewank global optimization problem. This 
    is a highly multimodal minimization problem with numerous, wide-spread, 
    regularly distributed local minima. It is defined as follows:
    
    .. math::
    
        f(x) = \\frac{1}{4000} \sum_{i=1}^n x_i^2 - \prod_{i=1}^n \cos(\\frac{x_i}{\sqrt{i}}) + 1
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-600, 600]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image8891.jpg
        :alt: Griewank function
        :align: center
        
        Two-dimensional Griewank function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page1905.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [0, 0, ..., 0].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-600.0] * self.dimensions, [600.0] * self.dimensions)
        self.maximize = False
        self.global_optimum = [0 for _ in range(self.dimensions)]
        
    def generator(self, random, args):
        return [random.uniform(-600.0, 600.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            fitness.append(1.0 / 4000.0 * sum([x**2 for x in c]) - 
                           reduce(lambda x, y: x*y, [math.cos(x / math.sqrt(i+1)) for i, x in enumerate(c)]) + 1)
        return fitness

class Rastrigin(Benchmark):
    """Defines the Rastrigin benchmark problem.
    
    This class defines the Rastrigin global optimization problem. This 
    is a highly multimodal minimization problem where the local minima
    are regularly distributed. It is defined as follows:
    
    .. math::
    
        f(x) = \sum_{i=1}^n (x_i^2 - 10\cos(2\pi x_i) + 10)
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-5.12, 5.12]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image12271.jpg
        :alt: Rastrigin function
        :align: center
        
        Two-dimensional Rastrigin function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page2607.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [0, 0, ..., 0].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-5.12] * self.dimensions, [5.12] * self.dimensions)
        self.maximize = False
        self.global_optimum = [0 for _ in range(self.dimensions)]
        
    def generator(self, random, args):
        return [random.uniform(-5.12, 5.12) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            fitness.append(sum([x**2 - 10 * math.cos(2 * math.pi * x) + 10 for x in c]))
        return fitness

class Rosenbrock(Benchmark):
    """Defines the Rosenbrock benchmark problem.
    
    This class defines the Rosenbrock global optimization problem, 
    also known as the "banana function." The global optimum sits 
    within a narrow, parabolic-shaped flattened valley. It is 
    defined as follows:
    
    .. math::
    
        f(x) = \sum_{i=1}^{n-1} [100(x_i^2 - x_{i+1})^2 + (x_i - 1)^2]
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-5, 10]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image12371.jpg
        :alt: Rosenbrock function
        :align: center
        
        Two-dimensional Rosenbrock function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page2537.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [1, 1, ..., 1].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-5.0] * self.dimensions, [10.0] * self.dimensions)
        self.maximize = False
        self.global_optimum = [1 for _ in range(self.dimensions)]
    
    def generator(self, random, args):
        return [random.uniform(-5.0, 10.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            total = 0
            for i in range(len(c) - 1):
                total += 100 * (c[i]**2 - c[i+1])**2 + (c[i] - 1)**2
            fitness.append(total)
        return fitness
    
class Schwefel(Benchmark):
    """Defines the Schwefel benchmark problem.
    
    This class defines the Schwefel global optimization problem. 
    It is defined as follows:
    
    .. math::
    
        f(x) = 418.9829n - \sum_{i=1}^n \\left[-x_i \sin(\sqrt{|x_i|})\\right]
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-500, 500]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image12721.jpg
        :alt: Schwefel function
        :align: center
        
        Two-dimensional Schwefel function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page2530.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [420.9687, 420.9687, ..., 420.9687].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-500.0] * self.dimensions, [500.0] * self.dimensions)
        self.maximize = False
        self.global_optimum = [420.9687 for _ in range(self.dimensions)]
    
    def generator(self, random, args):
        return [random.uniform(-500.0, 500.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            fitness.append(418.9829 * self.dimensions - sum([x * math.sin(math.sqrt(abs(x))) for x in c]))
        return fitness
    
class Sphere(Benchmark):
    """Defines the Sphere benchmark problem.
    
    This class defines the Sphere global optimization problem, also called
    the "first function of De Jong's" or "De Jong's F1." It is continuous,
    convex, and unimodal, and it is defined as follows:
    
    .. math::
    
        f(x) = \sum_{i=1}^n x_i^2
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-5.12, 5.12]` for :math:`i=1,...,n`.
    
    .. figure:: http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/image11981.jpg
        :alt: Sphere function
        :align: center
        
        Two-dimensional Sphere function 
        (`image source <http://www-optima.amp.i.kyoto-u.ac.jp/member/student/hedar/Hedar_files/TestGO_files/Page1113.htm>`__)
    
    Public Attributes:
    
    - *global_optimum* -- the problem input that produces the optimum output.
      Here, this corresponds to [0, 0, ..., 0].
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions)
        self.bounder = ec.Bounder([-5.12] * self.dimensions, [5.12] * self.dimensions)
        self.maximize = False
        self.global_optimum = [0 for _ in range(self.dimensions)]
    
    def generator(self, random, args):
        return [random.uniform(-5.12, 5.12) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            fitness.append(sum([x**2 for x in c]))
        return fitness
    


        
#------------------------------------------------------
#              Multi-objective problems
#------------------------------------------------------

class Kursawe(Benchmark):
    """Defines the Kursawe multiobjective benchmark problem.
    
    This class defines the Kursawe multiobjective minimization problem. 
    This function accepts an n-dimensional input and produces a 
    two-dimensional output. It is defined as follows:
    
    .. math::
    
        f_1(x) &= \sum_{i=1}^{n-1} \\left[-10e^{-0.2\sqrt{x_i^2 + x_{i+1}^2}}\\right] \\\\
        f_2(x) &= \sum_{i=1}^n \\left[|x_i|^{0.8} + 5\sin(x_i)^3\\right] \\\\
    
    Here, :math:`n` represents the number of dimensions and :math:`x_i \in [-5, 5]` for :math:`i=1,...,n`.
    
    .. figure:: http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/mopfigs/kursawefun.jpg
        :alt: Kursawe Pareto front
        :width: 700 px
        :align: center
        
        Three-dimensional Kursawe Pareto front 
        (`image source <http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/>`__)
    
    """
    def __init__(self, dimensions=2):
        Benchmark.__init__(self, dimensions, 2)
        self.bounder = ec.Bounder([-5.0] * self.dimensions, [5.0] * self.dimensions)
        self.maximize = False

    def generator(self, random, args):
        return [random.uniform(-5.0, 5.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        for c in candidates:
            f1 = sum([-10 * math.exp(-0.2 * math.sqrt(c[i]**2 + c[i+1]**2)) for i in range(len(c) - 1)])
            f2 = sum([math.pow(abs(x), 0.8) + 5 * math.sin(x)**3 for x in c])
            fitness.append(emo.Pareto([f1, f2]))
        return fitness

class DTLZ1(Benchmark):
    """Defines the DTLZ1 multiobjective benchmark problem.
    
    This class defines the DTLZ1 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= \\frac{1}{2} x_1 \\dots x_{m-1}(1 + g(\\vec{x_m})) \\\\
        f_i(\\vec{x}) &= \\frac{1}{2} x_1 \\dots x_{m-i}(1 + g(\\vec{x_m})) \\\\
        f_m(\\vec{x}) &= \\frac{1}{2} (1 - x_1)(1 + g(\\vec{x_m})) \\\\
        g(\\vec{x_m}) &= 100\\left[|\\vec{x_m}| + \sum_{x_i \in \\vec{x_m}}\\left((x_i - 0.5)^2 - \cos(20\pi(x_i - 0.5))\\right)\\right] \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 4 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 6 dimensions.
    
    .. figure:: http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/mopfigs/dtlz1funb.jpg
        :alt: DTLZ1 Pareto front
        :width: 700 px
        :align: center
        
        Three-dimensional DTLZ1 Pareto front 
        (`image source <http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/>`__)
        
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
        
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0 for _ in range(self.dimensions - self.objectives + 1)])
        return x
    
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: 100 * (len(x) + sum([(a - 0.5)**2 - math.cos(20 * math.pi * (a - 0.5)) for a in x]))
        for c in candidates:
            gval = g(c[self.objectives-1:])
            fit = [0.5 * reduce(lambda x,y: x*y, c[:self.objectives-1]) * (1 + gval)]
            for m in reversed(range(1, self.objectives)):
                fit.append(0.5 * reduce(lambda x,y: x*y, c[:m-1], 1) * (1 - c[m-1]) * (1 + gval))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ2(Benchmark):
    """Defines the DTLZ2 multiobjective benchmark problem.
    
    This class defines the DTLZ2 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1 \pi/2)\cos(x_2 \pi/2)\\dots\cos(x_{m-2} \pi/2)\cos(x_{m-1} \pi/2) \\\\
        f_i(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1 \pi/2)\cos(x_2 \pi/2)\\dots\cos(x_{m-i} \pi/2)\sin(x_{m-i+1} \pi/2) \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))\sin(x_1 \pi/2) \\\\
        g(\\vec{x_m}) &= \sum_{x_i \in \\vec{x_m}}(x_i - 0.5)^2 \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 9 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 11 dimensions.
    
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0.5 for _ in range(self.dimensions - self.objectives + 1)])
        return x
    
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: sum([(a - 0.5)**2 for a in x])
        for c in candidates:
            gval = g(c[self.objectives-1:])
            fit = [(1 + gval) * 
                   reduce(lambda x,y: x*y, [math.cos(a * math.pi / 2.0) for a in c[:self.objectives-1]])]
            for m in reversed(range(1, self.objectives)):
                fit.append((1 + gval) * 
                           reduce(lambda x,y: x*y, [math.cos(a * math.pi / 2.0) for a in c[:m-1]], 1) *
                           math.sin(c[m-1] * math.pi / 2.0))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ3(Benchmark):
    """Defines the DTLZ3 multiobjective benchmark problem.
    
    This class defines the DTLZ3 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1 \pi/2)\cos(x_2 \pi/2)\\dots\cos(x_{m-2} \pi/2)\cos(x_{m-1} \pi/2) \\\\
        f_i(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1 \pi/2)\cos(x_2 \pi/2)\\dots\cos(x_{m-i} \pi/2)\sin(x_{m-i+1} \pi/2) \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))\sin(x_1 \pi/2) \\\\
        g(\\vec{x_m}) &= 100\\left[|\\vec{x_m}| + \sum_{x_i \in \\vec{x_m}}\\left((x_i - 0.5)^2 - \cos(20\pi(x_i - 0.5))\\right)\\right] \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 9 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 11 dimensions.
    
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0.5 for _ in range(self.dimensions - self.objectives + 1)])
        return x
        
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: 100 * (len(x) + sum([(a - 0.5)**2 - math.cos(20 * math.pi * (a - 0.5)) for a in x]))
        for c in candidates:
            gval = g(c[self.objectives-1:])
            fit = [(1 + gval) * reduce(lambda x,y: x*y, [math.cos(a * math.pi / 2.0) for a in c[:self.objectives-1]])]
            for m in reversed(range(1, self.objectives)):
                fit.append((1 + gval) * 
                           reduce(lambda x,y: x*y, [math.cos(a * math.pi / 2.0) for a in c[:m-1]], 1) *
                           math.sin(c[m-1] * math.pi / 2.0))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ4(Benchmark):
    """Defines the DTLZ4 multiobjective benchmark problem.
    
    This class defines the DTLZ4 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1^\\alpha \pi/2)\cos(x_2^\\alpha \pi/2)\\dots\cos(x_{m-2}^\\alpha \pi/2)\cos(x_{m-1}^\\alpha \pi/2) \\\\
        f_i(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(x_1^\\alpha \pi/2)\cos(x_2^\\alpha \pi/2)\\dots\cos(x_{m-i}^\\alpha \pi/2)\sin(x_{m-i+1}^\\alpha \pi/2) \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))\sin(x_1^\\alpha \pi/2) \\\\
        g(\\vec{x_m}) &= \sum_{x_i \in \\vec{x_m}}(x_i - 0.5)^2 \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`,  
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n},` and :math:`\\alpha=100.`
    
    The recommendation given in the paper mentioned above is to provide 9 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 11 dimensions.
    
    """
    def __init__(self, dimensions=2, objectives=2, alpha=100):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
        self.alpha = alpha
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0.5 for _ in range(self.dimensions - self.objectives + 1)])
        return x
        
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: sum([(a - 0.5)**2 for a in x])
        for c in candidates:
            gval = g(c[self.objectives-1:])
            fit = [(1 + gval) * 
                   reduce(lambda x,y: x*y, [math.cos(a**self.alpha * math.pi / 2.0) for a in c[:self.objectives-1]])]
            for m in reversed(range(1, self.objectives)):
                fit.append((1 + gval) * 
                           reduce(lambda x,y: x*y, [math.cos(a**self.alpha * math.pi / 2.0) for a in c[:m-1]], 1) *
                           math.sin(c[m-1]**self.alpha * math.pi / 2.0))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ5(Benchmark):
    """Defines the DTLZ5 multiobjective benchmark problem.
    
    This class defines the DTLZ5 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(\\theta_1 \pi/2)\cos(\\theta_2 \pi/2)\\dots\cos(\\theta_{m-2} \pi/2)\cos(\\theta_{m-1} \pi/2) \\\\
        f_i(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(\\theta_1 \pi/2)\cos(\\theta_2 \pi/2)\\dots\cos(\\theta_{m-i} \pi/2)\sin(\\theta_{m-i+1} \pi/2) \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))\sin(\\theta_1 \pi/2) \\\\
        \\theta_i     &= \\frac{\pi}{4(1+g(\\vec{x_m}))}(1 + 2g(\\vec{x_m}) x_i) \\\\
        g(\\vec{x_m}) &= \sum_{x_i \in \\vec{x_m}}(x_i - 0.5)^2 \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 9 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 11 dimensions.
    
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0.5 for _ in range(self.dimensions - self.objectives + 1)])
        return x
        
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: sum([(a - 0.5)**2 for a in x])
        for c in candidates:
            gval = g(c[self.objectives-1:])
            theta = lambda x: math.pi / (4.0 * (1 + gval)) * (1 + 2 * gval * x)
            fit = [(1 + gval) * math.cos(math.pi / 2.0 * c[0]) *
                   reduce(lambda x,y: x*y, [math.cos(theta(a)) for a in c[1:self.objectives-1]])]
            for m in reversed(range(1, self.objectives)):
                if m == 1:
                    fit.append((1 + gval) * math.sin(math.pi / 2.0 * c[0]))
                else:
                    fit.append((1 + gval) * math.cos(math.pi / 2.0 * c[0]) *
                               reduce(lambda x,y: x*y, [math.cos(theta(a)) for a in c[1:m-1]], 1) *
                               math.sin(theta(c[m-1])))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ6(Benchmark):
    """Defines the DTLZ6 multiobjective benchmark problem.
    
    This class defines the DTLZ6 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(\\theta_1 \pi/2)\cos(\\theta_2 \pi/2)\\dots\cos(\\theta_{m-2} \pi/2)\cos(\\theta_{m-1} \pi/2) \\\\
        f_i(\\vec{x}) &= (1 + g(\\vec{x_m}))\cos(\\theta_1 \pi/2)\cos(\\theta_2 \pi/2)\\dots\cos(\\theta_{m-i} \pi/2)\sin(\\theta_{m-i+1} \pi/2) \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))\sin(\\theta_1 \pi/2) \\\\
        \\theta_i     &= \\frac{\pi}{4(1+g(\\vec{x_m}))}(1 + 2g(\\vec{x_m}) x_i) \\\\
        g(\\vec{x_m}) &= \sum_{x_i \in \\vec{x_m}}x_i^{0.1} \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 9 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 11 dimensions.
    
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0 for _ in range(self.dimensions - self.objectives + 1)])
        return x
        
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: sum([a**0.1 for a in x])
        for c in candidates:
            gval = g(c[self.objectives-1:])
            theta = lambda x: math.pi / (4.0 * (1 + gval)) * (1 + 2 * gval * x)
            fit = [(1 + gval) * math.cos(math.pi / 2.0 * c[0]) *
                   reduce(lambda x,y: x*y, [math.cos(theta(a)) for a in c[1:self.objectives-1]])]
            for m in reversed(range(1, self.objectives)):
                if m == 1:
                    fit.append((1 + gval) * math.sin(math.pi / 2.0 * c[0]))
                else:
                    fit.append((1 + gval) * math.cos(math.pi / 2.0 * c[0]) *
                               reduce(lambda x,y: x*y, [math.cos(theta(a)) for a in c[1:m-1]], 1) *
                               math.sin(theta(c[m-1])))
            fitness.append(emo.Pareto(fit))
        return fitness

class DTLZ7(Benchmark):
    """Defines the DTLZ7 multiobjective benchmark problem.
    
    This class defines the DTLZ7 multiobjective minimization problem.
    taken from `(Deb et al., "Scalable Multi-Objective Optimization Test Problems."
    CEC 2002, pp. 825--830) <http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/dtlz1/index.php>`__.
    This function accepts an n-dimensional input and produces an m-dimensional output.
    It is defined as follows:
    
    .. math::
    
        f_1(\\vec{x}) &= x_1 \\\\
        f_i(\\vec{x}) &= x_i \\\\
        f_m(\\vec{x}) &= (1 + g(\\vec{x_m}))h(f_1, f_2, \\dots, f_{m-1}, g) \\\\
        g(\\vec{x_m}) &= 1 + \\frac{9}{|\\vec{x_m}|}\sum_{x_i \in \\vec{x_m}}x_i \\\\
        h(f_1, f_2, \\dots, f_{m-1}, g) &= m - \sum_{i=1}^{m-1}\\left[\\frac{f_1}{1+g}(1 + \sin(3\pi f_i))\\right] \\\\
    
    Here, :math:`n` represents the number of dimensions, :math:`m` represents the
    number of objectives, :math:`x_i \in [0, 1]` for :math:`i=1,...,n`, and 
    :math:`\\vec{x_m} = x_m x_{m+1} \\dots x_{n}.`
    
    The recommendation given in the paper mentioned above is to provide 19 more
    dimensions than objectives. For instance, if we want to use 2 objectives, we
    should use 21 dimensions.
    
    .. figure:: http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/mopfigs/dtlz7funb.jpg
        :alt: DTLZ7 Pareto front
        :width: 700 px
        :align: center
        
        Three-dimensional DTLZ7 Pareto front 
        (`image source <http://delta.cs.cinvestav.mx/~ccoello/EMOO/testfuncs/>`__)
        
    """
    def __init__(self, dimensions=2, objectives=2):
        Benchmark.__init__(self, dimensions, objectives)
        if dimensions < objectives:
            raise ValueError('dimensions (%d) must be greater than or equal to objectives (%d)' % (dimensions, objectives))
        self.bounder = ec.Bounder([0.0] * self.dimensions, [1.0] * self.dimensions)
        self.maximize = False
    
    def global_optimum(self):
        x = [random.uniform(0, 1) for _ in range(self.objectives - 1)]
        x.extend([0 for _ in range(self.dimensions - self.objectives + 1)])
        return x
        
    def generator(self, random, args):
        return [random.uniform(0.0, 1.0) for _ in range(self.dimensions)]
        
    def evaluator(self, candidates, args):
        fitness = []
        g = lambda x: 1 + 9.0 / len(x) * sum([a for a in x])
        for c in candidates:
            gval = g(c[self.objectives-1:])
            fit = []
            for m in range(self.objectives-1):
                fit.append(c[m])
            fit.append((1 + gval) * (self.objectives - sum([a / (1.0 + gval) * (1 + math.sin(3 * math.pi * a)) for a in c[:self.objectives-1]])))
            fitness.append(emo.Pareto(fit))
        return fitness



    






import unittest
import random
import logging
import itertools
from ecspy import ec
from ecspy import emo
from ecspy import archivers
from ecspy import evaluators
from ecspy import migrators
from ecspy import replacers
from ecspy import selectors
from ecspy import terminators
from ecspy import variators


def test_generator(random, args):
    return [random.random() for _ in range(6)]
    
def test_evaluator(candidates, args):
    fitness = []
    for c in candidates:
        fitness.append(sum(c))
    return fitness

def test_multiobjective_evaluator(candidates, args):
    fitness = []
    for c in candidates:
        fitness.append(emo.Pareto([sum(c), sum(c)]))
    return fitness
    
prng = random.Random()
prng.seed(111111)
test_candidates = [test_generator(prng, {}) for _ in range(12)]
test_fitnesses = test_evaluator(test_candidates, {})
test_multiobjective_fitnesses = test_multiobjective_evaluator(test_candidates, {})
test_population = [ec.Individual(candidate=c) for c in test_candidates]
test_multiobjective_population = [ec.Individual(candidate=c) for c in test_candidates]
for i, f in zip(test_population, test_fitnesses):
    i.fitness = f
test_parents = test_population[:6]
test_offspring = test_population[6:]
for i, f in zip(test_multiobjective_population, test_multiobjective_fitnesses):
    i.fitness = f
test_multiobjective_parents = test_multiobjective_population[:6]
test_multiobjective_offspring = test_multiobjective_population[6:]
    

class ArchiverTests(unittest.TestCase):
    def test_default_archiver(self):
        new_archive = archivers.default_archiver(prng, test_population, [], {})
        assert new_archive == test_population
        
    def test_best_archiver(self):
        new_archive = archivers.best_archiver(prng, test_population, [], {})
        assert new_archive == [max(test_population)]
        
    def test_adaptive_grid_archiver(self):
        new_archive = archivers.adaptive_grid_archiver(prng, test_multiobjective_population, [], {})
        assert len(new_archive) == 1
        
class EvaluatorTests(unittest.TestCase):
    def test_parallel_evaluation_pp(self):
        class fake_ec(object):
            def __init__(self):
                self.logger = logging.getLogger('ecspy.test')
        x = fake_ec()
        fitnesses = evaluators.parallel_evaluation_pp(test_candidates, {'_ec':x, 'pp_evaluator':test_evaluator})
        assert fitnesses == test_fitnesses
        
    def test_parallel_evaluation_mp(self):
        class fake_ec(object):
            def __init__(self):
                self.logger = logging.getLogger('ecspy.test')
        x = fake_ec()
        fitnesses = evaluators.parallel_evaluation_mp(test_candidates, {'_ec':x, 'mp_evaluator':test_evaluator})
        assert fitnesses == test_fitnesses
        
class MigratorTests(unittest.TestCase):
    def test_default_migration(self):
        migrants = migrators.default_migration(prng, test_population, {})
        assert migrants == test_population
        
class ReplacerTests(unittest.TestCase):
    def test_default_replacement(self):
        survivors = replacers.default_replacement(prng, test_population, test_parents, test_offspring, {})
        assert survivors == test_offspring
        
    def test_truncation_replacement(self):
        survivors = replacers.truncation_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == len(test_population) and max(max(test_population), max(test_offspring)) == max(survivors)
        
    def test_steady_state_replacement(self):
        survivors = replacers.steady_state_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == len(test_population) and all([o in survivors for o in test_offspring])
        
    def test_generational_replacement(self):
        survivors = replacers.generational_replacement(prng, test_population, test_parents, test_offspring, {})
        assert all([s in test_offspring for s in survivors])
        
    def test_random_replacement(self):
        survivors = replacers.random_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == len(test_population) and all([o in survivors for o in test_offspring])
    
    def test_plus_replacement(self):
        survivors = replacers.plus_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == len(test_population) and max(max(test_parents), max(test_offspring)) == max(survivors)
    
    def test_comma_replacement(self):
        survivors = replacers.comma_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == min(len(test_population), len(test_offspring)) and all([s in test_offspring for s in survivors])
    
    def test_crowding_replacement(self):
        survivors = replacers.crowding_replacement(prng, test_population, test_parents, test_offspring, {})
        assert len(survivors) == len(test_population) and max(max(test_population), max(test_offspring)) == max(survivors)
    
    def test_simulated_annealing_replacement(self):
        class fake_ec(object):
            def __init__(self):
                self.num_evaluations = 10
        x = fake_ec()
        survivors = replacers.simulated_annealing_replacement(prng, test_population, test_parents, test_offspring, 
                                                              {'_ec':x, 'max_evaluations':100})
        assert len(survivors) == len(test_parents) and max(max(test_parents), max(test_offspring)) == max(survivors)
    
    def test_nsga_replacement(self):
        survivors = replacers.nsga_replacement(prng, test_multiobjective_population, test_multiobjective_parents, test_multiobjective_offspring, {})
        assert (len(survivors) == len(test_multiobjective_population) and 
                max(max(test_multiobjective_population), max(test_multiobjective_offspring)) == max(survivors))
    
    def test_paes_replacement(self):
        class fake_ec(object):
            def __init__(self):
                self.archive = []
                self.archiver = archivers.adaptive_grid_archiver
        x = fake_ec()
        survivors = replacers.paes_replacement(prng, test_multiobjective_population, test_multiobjective_parents, 
                                               test_multiobjective_offspring, {'_ec':x})
        assert (len(survivors) == min(len(test_multiobjective_parents), len(test_multiobjective_offspring)) and 
                max(survivors) == max(max(test_multiobjective_parents), max(test_multiobjective_offspring)))
    
class SelectorTests(unittest.TestCase):
    def test_default_selection(self):
        parents = selectors.default_selection(prng, test_population, {})
        assert parents == test_population

    def test_truncation_selection(self):
        parents = selectors.truncation_selection(prng, test_population, {})
        assert all([p in parents for p in test_population])

    def test_uniform_selection(self):
        parents = selectors.uniform_selection(prng, test_population, {})
        assert len(parents) == 1 and all([p in test_population for p in parents])

    def test_fitness_proportionate_selection(self):
        parents = selectors.fitness_proportionate_selection(prng, test_population, {})
        assert len(parents) == 1 and all([p in test_population for p in parents])

    def test_rank_selection(self):
        parents = selectors.rank_selection(prng, test_population, {})
        assert len(parents) == 1 and all([p in test_population for p in parents])

    def test_tournament_selection(self):
        parents = selectors.tournament_selection(prng, test_population, {'tourn_size':len(test_population)})
        assert len(parents) == 1 and max(parents) == max(test_population)

class TerminatorTests(unittest.TestCase):
    def test_default_termination(self):
        t = terminators.default_termination(test_population, 1, 1, {})
        assert t == True

    def test_diversity_termination(self):
        p = [ec.Individual(candidate=[1, 1, 1]) for _ in range(10)]
        t = terminators.diversity_termination(p, 1, 1, {})
        assert t == True

    def test_average_fitness_termination(self):
        p = [ec.Individual(candidate=i.candidate) for i in test_population]
        for x in p:
            x.fitness = 1
        t = terminators.average_fitness_termination(p, 1, 1, {})
        assert t == True

    def test_evaluation_termination(self):
        t = terminators.evaluation_termination(test_population, 1, len(test_population), {})
        assert t == True

    def test_generation_termination(self):
        t = terminators.generation_termination(test_population, 1, 1, {})
        assert t == True

    def test_time_termination(self):
        class fake_ec(object):
            def __init__(self):
                self.logger = logging.getLogger('ecspy.test')
        x = fake_ec()
        t = terminators.time_termination(test_population, 1, 1, {'_ec':x, 'max_time':0})
        assert t == True

class VariatorTests(unittest.TestCase):
    def test_default_variation(self):
        offspring = variators.default_variation(prng, list(test_candidates), {})
        assert offspring == test_candidates

    def test_estimation_of_distribution_variation(self):
        class fake_ec(object):
            def __init__(self):
                self.bounder = ec.Bounder()
        x = fake_ec()
        test_candidates = [[1] * 6 for _ in range(10)]
        offspring = variators.estimation_of_distribution_variation(prng, list(test_candidates), {'_ec':x})
        assert len(offspring) == 1 and offspring[0] == test_candidates[0]

    def test_n_point_crossover(self):
        offspring = variators.n_point_crossover(prng, list(test_candidates), {'num_crossover_points':3})
        moms = test_candidates[::2]
        dads = test_candidates[1::2]
        dmoms = itertools.chain.from_iterable([[t, t] for t in moms])
        ddads = itertools.chain.from_iterable([[t, t] for t in dads])
        offs = [(offspring[i], offspring[i+1]) for i in range(0, len(offspring), 2)]
        assert (all([x in m or x in d for m, d, o in zip(dmoms, ddads, offspring) for x in o]) and 
                all([(x in o[0] or x in o[1]) and (y in o[0] or y in o[1]) for m, d, o in zip(moms, dads, offs) for x in m for y in m]))

    def test_uniform_crossover(self):
        offspring = variators.uniform_crossover(prng, list(test_candidates), {})
        moms = test_candidates[::2]
        dads = test_candidates[1::2]
        dmoms = itertools.chain.from_iterable([[t, t] for t in moms])
        ddads = itertools.chain.from_iterable([[t, t] for t in dads])
        offs = [(offspring[i], offspring[i+1]) for i in range(0, len(offspring), 2)]
        assert (all([x in m or x in d for m, d, o in zip(dmoms, ddads, offspring) for x in o]) and 
                all([(x in o[0] or x in o[1]) and (y in o[0] or y in o[1]) for m, d, o in zip(moms, dads, offs) for x in m for y in m]))
    
    def test_blend_crossover(self):
        class fake_ec(object):
            def __init__(self):
                self.bounder = ec.Bounder()
        x = fake_ec()
        alpha = 0.1
        offspring = variators.blend_crossover(prng, list(test_candidates), {'_ec':x, 'blx_alpha':alpha})
        moms = itertools.chain.from_iterable([[t, t] for t in test_candidates[::2]])
        dads = itertools.chain.from_iterable([[t, t] for t in test_candidates[1::2]])
        tests = []
        for mom, dad, off in zip(moms, dads, offspring):
            for m, d, x in zip(mom, dad, off):
                tol = alpha * (max(m, d) - min(m, d))
                tests.append(x >= (min(m, d) - tol) and x <= (max(m, d) + tol))
        assert all(tests)
    
    def test_differential_crossover(self):
        class fake_ec(object):
            def __init__(self):
                self.bounder = ec.Bounder()
                self.population = list(test_population)
        x = fake_ec()
        offspring = variators.differential_crossover(prng, list(test_candidates), {'_ec':x})
        moms = itertools.chain.from_iterable([[t, t] for t in test_candidates[::2]])
        dads = itertools.chain.from_iterable([[t, t] for t in test_candidates[1::2]])
        tests = []
        for mom, dad, off in zip(moms, dads, offspring):
            for m, d, x in zip(mom, dad, off):
                tests.append(x >= min(m, d) and x <= max(m, d))
        assert all(tests)
    
    def test_simulated_binary_crossover(self):
        assert True
    
    def test_laplace_crossover(self):
        assert True
    
    def test_gaussian_mutation(self):
        class fake_ec(object):
            def __init__(self):
                self.bounder = ec.Bounder(0, 1)
        x = fake_ec()
        offspring = variators.gaussian_mutation(prng, list(test_candidates), {'_ec':x})
        assert([x >= 0 and x <= 1 for o in offspring for x in o])
        
    def test_bit_flip_mutation(self):
        assert True
    
    def test_nonuniform_mutation(self):
        assert True
    
    def test_mptm_mutation(self):
        assert True
    



    
if __name__ == '__main__':
    unittest.main()
        
        
        
        
        


'''
Created on Jul 26, 2010

symbolic regression of a 6th order polynominal using cartesian genetic programming

'''
__author__ = 'Henrik Rudstrom'
from ecspy.variators.crossovers import n_point_crossover
from ecspy.contrib.cgp import CGPPhenotype, CGPEncoding


import ecspy
from sympy.core.symbol import Symbol
from sympy.core.numbers import RealNumber, Number


from random import Random
import random
from time import time
from ecspy import ec, terminators
import math
from ecspy.selectors import tournament_selection



#Node functions
def add(a,b):
    return a + b
def sub(a,b):
    return a - b
def mul(a,b):
    return a * b
def div(a,b):
    '''
    return numerator if denominator is 0
    '''
    return a if b == 0.0 else a / b


def get_fitness_function(target_function, constants):
    '''

    The fitness cases were 50 random values of X from the interval [-1.0, +1.0].
    These were fixed throughout the evolutionary run. The fitness of a program was
    defined as the sum over the 50 fitness cases of the sum of the absolute value of
    the error between the value returned by the program and that corresponding to the
    sixth order polynomial.
    --Miller
    '''
    sample_points = list([random.random()*2-1 for _ in xrange(50)])
    def fitness(candidates, args):
        fitness = []
         #sample_points = list([random.random()*2-1 for _ in xrange(50)])
        for cand in candidates:
            fit = 0.0
            func = CGPPhenotype(encoding, cand)
               
            for x in sample_points:
                target = target_function(x)
                
                solution = func([x]+constants)[0]
                fit += math.sqrt(abs(target - solution))

            fitness.append(fit)
        return fitness
    return fitness

def screen_observer(enc):
    def screen_observer(population, num_generations, num_evaluations, args):
        """Print the output of the EC to the screen.
        
        This function displays the results of the evolutionary computation
        to the screen. The output includes the generation number, the current
        number of evaluations, the average fitness, the maximum fitness, and 
        the full population.
        
        .. Arguments:
           population -- the population of Individuals
           num_generations -- the number of elapsed generations
           num_evaluations -- the number of candidate solution evaluations
           args -- a dictionary of keyword arguments
        
        """
        import numpy
        
        population = list(population)
        population.sort(reverse=True)
        worst_fit = population[-1].fitness
        best_fit = population[0].fitness
        med_fit = numpy.median([p.fitness for p in population])
        avg_fit = numpy.mean([p.fitness for p in population])
        std_fit = numpy.std([p.fitness for p in population], ddof=1)
        if(num_generations % 50 != 0):
            return
        print('Generation Evaluation Worst      Best       Median     Average    Std Dev   ')
        print('---------- ---------- ---------- ---------- ---------- ---------- ----------')
        print('{0:10} {1:10} {2:10} {3:10} {4:10} {5:10} {6:10}\n'.format(num_generations, num_evaluations, worst_fit, best_fit, med_fit, avg_fit, std_fit))
        print('Current Population:')
        print('----------------------------------------------------------------------------')
        for i, c in enumerate(population):
            f = CGPPhenotype(enc, c.candidate)
            exp = f([Symbol('x'), RealNumber(1.0)])
            print i, c.fitness, exp 
            
    return screen_observer   
        
def generate_initial_population(generator, n, sel, fitness_function):
    indexes = list(xrange(n))
    candidates = [generator(None, None) for _ in indexes]
    fitness = fitness_function(candidates, {})
    indexes.sort(key=lambda i: fitness[i])
    return [candidates[i] for i in indexes][:sel]
        
            
if __name__ == '__main__':
    constants = [1.]
    encoding = CGPEncoding(10, 2, 10, [add, sub, mul, div], 1)

    target = lambda x: x ** 6 - 2 * x ** 4 + x ** 2
    fitness_function = get_fitness_function(target, constants)


    mutator = encoding.mutator()
    generator = encoding.generator()
    seeds = generate_initial_population(generator, 500, 4, fitness_function)

    prng = Random()
    prng.seed(time())    
    ga = ec.ES(prng)

    ga.observer = [screen_observer(encoding)]
    ga.terminator = terminators.evaluation_termination
    ga.variator = [mutator]
    #ga.selector = tournament_selection
    start = time()
    print "setup", seeds

    
    final_pop = ga.evolve(seeds = seeds,
                          evaluator=fitness_function,
                          generator=generator,
                          max_evaluations=80000, 
                          pop_size=5,
                          mutation_rate=0.2,
                          maximize=False,
                          use_one_fifth_rule=True)
    stop = time()

    print('***********************************')
    print('Total Execution Time: %0.5f seconds' % (stop - start))



import math

class approximate(object):
    def __init__(self, alpha=0.75, beta=1, gamma=1, max_granules=200):
        self.granules = []
        self.threshold = float('infinity')
        self.alpha = alpha
        self.beta = beta
        self.gamma = gamma
        self.max_granules = max_granules
        
    def _add_granule(self, candidate, fitness):
        if len(self.granules) >= self.max_granules:
            self.granules.remove(min(self.granules, key=lambda x: x[3]))
        self.granules.append([candidate, fitness, self.gamma * 1.0 / (math.exp(fitness)**self.beta), 1])
    
    def _membership(self, x, center, spread):
        return math.exp(-(x - center)**2 / spread**2)
    
    def __call__(self, f):
        def approximate_f(candidates, args):
            fitness = []
            for candidate in candidates:
                if len(self.granules) == 0:
                    val = f([candidate], args)[0]
                    self._add_granule(candidate, val)
                    fitness.append(val)
                else:
                    similarity = []
                    for granule in self.granules:
                        similarity.append(sum([self._membership(x, c, granule[2]) / float(len(granule[0])) for x, c in zip(candidate, granule[0])]))
                    m = max(similarity)
                    if m > self.threshold:
                        sim_index = similarity.index(m)
                        self.granules[sim_index][3] += 1
                        fitness.append(self.granules[sim_index][1])
                    else:
                        val = f([candidate], args)[0]
                        self._add_granule(candidate, val)
                        fitness.append(val)
            avg_fit = sum(fitness) / float(len(fitness))
            self.threshold = self.alpha * max(fitness) / avg_fit
            return fitness
        return approximate_f

    def __repr__(self):
        return self.func.__doc__


        

if __name__ == '__main__':
    import random
    import math
    import time
    from ecspy import ec
    from ecspy import observers
    from ecspy import terminators


    def rastrigin_generator(random, args):
        return [random.uniform(-5.12, 5.12) for _ in range(20)]

    @approximate(alpha=0.75, beta=0.004, gamma=0.15)
    def rastrigin_evaluator(candidates, args):
        fitness = []
        for cand in candidates:
            fitness.append(10 * len(cand) + sum([x**2 - 10 * (math.cos(2*math.pi*x)) for x in cand]))
        return fitness
        
    prng = random.Random()
    prng.seed(time.time())
    es = ec.ES(prng)
    #es.observer = observers.screen_observer
    es.terminator = terminators.evaluation_termination
    final_pop = es.evolve(rastrigin_generator, rastrigin_evaluator, pop_size=20, maximize=False, 
                          max_evaluations=5000, lower_bound=-5.12, upper_bound=5.12)

    for p in final_pop:
        print p



#start_imports
from random import Random
from time import time
from math import cos
from math import pi
from ecspy import ec
from ecspy import terminators
#end_imports


def generate_rastrigin(random, args):
    size = args.get('num_inputs', 10)
    return [random.uniform(-5.12, 5.12) for i in xrange(size)]

def evaluate_rastrigin(candidates, args):
    fitness = []
    for cs in candidates:
        fit = 10 * len(cs) + sum([((x - 1)**2 - 10 * cos(2 * pi * (x - 1))) for x in cs])
        fitness.append(fit)
    return fitness

#start_main
rand = Random()
rand.seed(int(time()))
es = ec.ES(rand)
es.terminator = terminators.evaluation_termination
final_pop = es.evolve(generator=generate_rastrigin,
                      evaluator=evaluate_rastrigin,
                      pop_size=100,
                      maximize=False,
                      bounder=ec.Bounder(-5.12, 5.12),
                      max_evaluations=20000,
                      mutation_rate=0.25,
                      num_inputs=3)
# Sort and print the best individual, who will be at index 0.
final_pop.sort(reverse=True)
print(final_pop[0])
#end_main


"""
    This module provides pre-defined archivers for evolutionary computations.
    
    All archiver functions have the following arguments:
    
    - *random* -- the random number generator object
    - *population* -- the population of individuals
    - *archive* -- the current archive of individuals
    - *args* -- a dictionary of keyword arguments
    
    Each archiver function returns the updated archive.
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math


def default_archiver(random, population, archive, args):
    """Archive the current population.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       archive -- the current archive of individuals
       args -- a dictionary of keyword arguments
    
    """
    new_archive = []
    for ind in population:
        new_archive.append(ind)
    return new_archive
    

def best_archiver(random, population, archive, args):
    """Archive only the best individual(s).
    
    This function archives the best solutions and removes inferior ones.
    If the comparison operators have been overloaded to define Pareto
    preference (as in the ``Pareto`` class), then this archiver will form 
    a Pareto archive.
    
    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       archive -- the current archive of individuals
       args -- a dictionary of keyword arguments
    
    """
    new_archive = archive
    for ind in population:
        if len(new_archive) == 0:
            new_archive.append(ind)
        else:
            should_remove = []
            should_add = True
            for a in new_archive:
                if ind == a:
                    should_add = False
                    break
                elif ind < a:
                    should_add = False
                elif ind > a:
                    should_remove.append(a)
            for r in should_remove:
                new_archive.remove(r)
            if should_add:
                new_archive.append(ind)
    return new_archive

    
def adaptive_grid_archiver(random, population, archive, args):
    """Archive only the best individual(s) using a fixed size grid.
    
    This function archives the best solutions by using a fixed-size grid
    to determine which existing solutions should be removed in order to
    make room for new ones. This archiver is designed specifically for
    use with the Pareto Archived Evolution Strategy (PAES).

    .. Arguments:
       random -- the random number generator object
       population -- the population of individuals
       archive -- the current archive of individuals
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *max_archive_size* -- the maximum number of individuals in the archive
      (default len(population))
    - *num_grid_divisions* -- the number of grid divisions (default 1)
    
    """
    def get_grid_location(fitness, num_grid_divisions, global_smallest, global_largest):
        loc = 0
        n = 1
        num_objectives = len(fitness)
        inc = [0 for _ in range(num_objectives)]
        width = [0 for _ in range(num_objectives)]
        local_smallest = global_smallest[:]
        for i, f in enumerate(fitness):
            if f < local_smallest[i] or f > local_smallest[i] + global_largest[i] - global_smallest[i]:
                return -1
        for i in range(num_objectives):
            inc[i] = n
            n *= 2
            width[i] = global_largest[i] - global_smallest[i]
        for d in range(num_grid_divisions):
            for i, f in enumerate(fitness):
                if f < width[i] / 2.0 + local_smallest[i]:
                    loc += inc[i]
                else:
                    local_smallest[i] += width[i] / 2.0
            for i in range(num_objectives):
                inc[i] *= num_objectives * 2
                width[i] /= 2.0
        return loc
 
    def update_grid(individual, archive, num_grid_divisions, global_smallest, global_largest, grid_population):
        if len(archive) == 0:
            num_objectives = len(individual.fitness)
            smallest = [individual.fitness[o] for o in range(num_objectives)]
            largest = [individual.fitness[o] for o in range(num_objectives)]
        else:
            num_objectives = min(min([len(a.fitness) for a in archive]), len(individual.fitness))
            smallest = [min(min([a.fitness[o] for a in archive]), individual.fitness[o]) for o in range(num_objectives)] 
            largest = [max(max([a.fitness[o] for a in archive]), individual.fitness[o]) for o in range(num_objectives)]
        for i in range(num_objectives):
            global_smallest[i] = smallest[i] - abs(0.2 * smallest[i])
            global_largest[i] = largest[i] + abs(0.2 * largest[i])
        for i in range(len(grid_population)):
            grid_population[i] = 0
        for a in archive:
            loc = get_grid_location(a.fitness, num_grid_divisions, global_smallest, global_largest)
            a.grid_location = loc
            grid_population[loc] += 1
        loc = get_grid_location(individual.fitness, num_grid_divisions, global_smallest, global_largest)
        individual.grid_location = loc
        grid_population[loc] += 1

    max_archive_size = args.setdefault('max_archive_size', len(population))
    num_grid_divisions = args.setdefault('num_grid_divisions', 1)
        
    if not 'grid_population' in dir(adaptive_grid_archiver):
        adaptive_grid_archiver.grid_population = [0 for _ in range(2**(min([len(p.fitness) for p in population]) * num_grid_divisions))]
    if not 'global_smallest' in dir(adaptive_grid_archiver):
        adaptive_grid_archiver.global_smallest = [0 for _ in range(min([len(p.fitness) for p in population]))]
    if not 'global_largest' in dir(adaptive_grid_archiver):
        adaptive_grid_archiver.global_largest = [0 for _ in range(min([len(p.fitness) for p in population]))]
     
    new_archive = archive
    for ind in population:
        update_grid(ind, new_archive, num_grid_divisions, adaptive_grid_archiver.global_smallest, 
                    adaptive_grid_archiver.global_largest, adaptive_grid_archiver.grid_population)
        should_be_added = True
        for a in new_archive:
            if ind == a or a > ind:
                should_be_added = False
                
        if should_be_added:
            if len(new_archive) == 0:
                new_archive.append(ind)
            else:
                join = False
                nondominated = True
                removal_set = []
                for i, a in enumerate(new_archive):
                    if ind > a and not join:
                        new_archive[i] = ind
                        join = True
                    elif ind > a:
                        if not a in removal_set: 
                            removal_set.append(a)
                    # Otherwise, the individual is nondominated against this archive member.
                    
                new_archive = list(set(new_archive) - set(removal_set))
                
                if not join and nondominated:
                    if len(new_archive) == max_archive_size:
                        replaced_index = 0
                        found_replacement = False
                        loc = get_grid_location(ind.fitness, num_grid_divisions, 
                                                adaptive_grid_archiver.global_smallest, 
                                                adaptive_grid_archiver.global_largest)
                        ind.grid_location = loc
                        if ind.grid_location >= 0:
                            most = adaptive_grid_archiver.grid_population[ind.grid_location]
                        else:
                            most = -1
                        for i, a in enumerate(new_archive):
                            pop_at_a = adaptive_grid_archiver.grid_population[a.grid_location]
                            if pop_at_a > most:
                                most = pop_at_a
                                replaced_index = i
                                found_replacement = True
                        if found_replacement:
                            new_archive[replaced_index] = ind
                    else:
                        new_archive.append(ind)
    return new_archive




class memoized(object):
    """Cache a function's return value each time it is called.
    
    This function serves as a function decorator to provide a caching of
    evaluated fitness values. If called later with the same arguments, 
    the cached value is returned instead of being re-evaluated.
    
    This decorator assumes that candidates can be converted into 
    ``tuple`` objects to be used for hashing into a dictionary. It 
    should be used when evaluating an *expensive* fitness 
    function to avoid costly re-evaluation of those fitnesses. The 
    typical usage is as follows::
    
        @memoized
        def expensive_fitness_function(candidates, args):
            # Implementation of expensive fitness calculation
    
    """
    def __init__(self, target):
        self.target = target
        self.cache = {}
        
    def __getattr__(self, name):
        if name == '__name__':
            return self.target.__name__
        else:
            return getattr(self, name)

    def __repr__(self):
        return self.target.__doc__
        
    def __call__(self, candidates, args):
        fitness = []
        for candidate in candidates:
            try:
                fitness.append(self.cache[tuple(candidate)])
            except KeyError:
                fitness.append(self.target([candidate], args)[0])
                self.cache[tuple(candidate)] = fitness[-1]
            except TypeError:
                fitness.append(self.target([candidate], args)[0])
        return fitness
        


import inspect
class instantiated(object):
    """Create a instantiated version of a function.
    
    This function allows an ordinary function passed to it to act 
    very much like a callable instance of a class. For ``ecspy``, 
    this means that evolutionary operators (selectors, variators,
    replacers, etc.) can be created as normal functions and then
    be given the ability to have attributes *that are specific to
    the instance*. Python functions can always have attributes without
    employing any special mechanism, but those attributes exist for the 
    function, and there is no way to create a new "instance" except
    by implementing a new function with the same functionality.
    This class provides a way to "instantiate" the same function
    multiple times in order to provide each "instance" with its own
    set of independent attributes.
    
    The attributes that are created on a instantiated function are
    passed into that function via the ubiquitous ``args`` variable
    in ``ecspy``. Any user-specified attributes are added to the 
    ``args`` dictionary and replace any existing entry if necessary.
    If the function modifies those entries in the dictionary (e.g.,
    when dynamically modifying parameters), the corresponding 
    attributes are modified as well.
    
    Note that this class makes heavy use of run-time inspection
    provided by the ``inspect`` core library. 
    
    The typical usage is as follows::
    
        def typical_function(*args, **kwargs):
            # Implementation of typical function
        
        fun_one = instantiated(typical_function)
        fun_two = instantiated(typical_function)
        fun_one.attribute = value_one
        fun_two.attribute = value_two
    
    """
    def __init__(self, target):
        self.target = target
        
    def __getattr__(self, name):
        if name == '__name__':
            return self.target.__name__
        else:
            return getattr(self, name)
        
    def __repr__(self):
        return self.target.__doc__
        
    def __call__(self, *args, **kwargs):
        params = self.__parameters()
        try:
            orig_args = dict(kwargs['args'])
            orig_args.update(params)
            newkwargs = dict(kwargs)
            newkwargs['args'] = orig_args
            newargs = args
        except KeyError:
            orig_args = dict(args[-1])
            orig_args.update(params)
            newargs = list(args[:-1])
            newargs.append(orig_args)
            newargs = tuple(newargs)
            newkwargs = kwargs
        return_value = self.target(*newargs, **newkwargs)
        local_dict = self.__find_self().__dict__
        try:
            for key in newkwargs['args']:
                if key in local_dict:
                    local_dict[key] = newkwargs['args'][key]
        except KeyError:
            for key in newargs[-1]:
                if key in local_dict:
                    local_dict[key] = newargs[-1][key]
        return return_value
        
    def __find_self(self):
        for i, entry in enumerate(inspect.stack()):
            locals = entry[0].f_locals
            for var in locals:
                if isinstance(locals[var], self.__class__):
                    return locals[var]
        raise TypeError('%s class is not being used' % self.__class__.__name__)
        
    def __parameters(self):
        found_self = self.__find_self()
        params = dict(found_self.__dict__)
        try:
            del params['target']
        except KeyError:
            pass
        return params
        


from random import Random
from time import time
from ecspy import ec
from ecspy import terminators, observers, replacers, selectors, variators, archivers

#sentence = """
#Methinks it is like a weasel.
#It is backed like a weasel.
#Or like a whale?
#Very like a whale
#"""

sentence = """
'Just living is not enough,' said the butterfly,
'one must have sunshine, freedom, and a little flower.'
"""

ddd = map(ord, sentence)
max_ord = max(ddd)
min_ord = min(ddd)
numeric_sentence = map(ord, sentence)

def optimal_termination(population, num_generations, num_evaluations, args):
    if any([i.fitness==0 for i in population]):
        return  True
    return False

def observer(population, num_generations, num_evaluations, args):
    if num_generations % 100 == 0:
        population = list(population)
        population.sort(reverse=True)
        best_fit = population[0].candidate
        print '*'*20
        print  ''.join(map(chr, best_fit))
        print 'fitnesses:',
        print 'n archive:', len(args['_ec'].archive)
        print 'n population:', len(population)
        print 'n evals:', num_evaluations
    
def generate_sentence(random, args):
    s = [random.randint(0, max_ord) for i in sentence]
    return s 
    
def evaluate_sentence(candidates, args):
    fitness = []
    for cand in candidates:
        fitt = sum([abs(a-b) for a,b in zip(cand, numeric_sentence)])
        fitness.append(fitt)
    return fitness

def integer_mutation(random, candidates, args):
    mut_rate = args.setdefault('mutation_rate', 0.1)
    step = args.setdefault('mutation_step', 3)
    cands = list(candidates)
    for i, cs in enumerate(cands):
        for j, c in enumerate(cs):
            if random.random() < mut_rate:
                s = random.randint(0, 2 * step + 1) - step
                cands[i][j] = c + s
    return cands


def main(prng=None):
    if prng is None:
        prng = Random()
        prng.seed(time())
            
    ta = time()
    ea = ec.EvolutionaryComputation(prng)
    ea.observer = observer
    ea.selector = selectors.tournament_selection
    ea.variator = [variators.uniform_crossover, integer_mutation]
    ea.replacer = replacers.comma_replacement
    ea.archiver = archivers.best_archiver
    ea.terminator = [optimal_termination, terminators.evaluation_termination]
    final_pop = ea.evolve(evaluator=evaluate_sentence,
                          generator=generate_sentence,
                          max_evaluations=100000, 
                          pop_size=60,
                          tourn_size=20,
                          num_selected=60, 
                          mutation_rate=0.01,
                          crossover_rate=1.0,
                          maximize=False)
    final_pop.sort(reverse=True)
    final_sentence= ''.join(map(chr, final_pop[0].candidate))
    print 'FINAL SENTENCE'
    print final_sentence
    print 'n generations:',ea.num_generations
    print 'n archive:', len(ea.archive)
    print 'n population:', len(ea.population)
    print 'evolution took:', time() - ta
    return ea
            
if __name__ == '__main__':
    TEST_PERFORMANCE = False
    if not TEST_PERFORMANCE:
        try:
            # 3 fold speed up!
            import psyco; psyco.full()
        except ImportError:
            pass
        main()
    else:
        import hotshot, hotshot.stats, test.pystone
        prof = hotshot.Profile("inf_monkey.prof")
        prof.runcall( main )
        prof.close()
        stats = hotshot.stats.load("inf_monkey.prof")
        stats.strip_dirs()
        stats.sort_stats('time', 'calls')
        stats.print_stats(20)
    

#===============================================================================
# 2760600/25800    4.394    0.000    7.778    0.000 copy.py:144(deepcopy)
#    25800    2.113    0.000    7.679    0.000 copy.py:223(_deepcopy_list)
#      430    1.471    0.003    1.495    0.003 variators.py:122(uniform_crossover)
#      430    1.329    0.003    1.401    0.003 infinite_monkey_theorem.py:43(gaussian_mutation)
#      431    1.261    0.003    1.261    0.003 infinite_monkey_theorem.py:36(evaluate_sentence)
#  1108827    0.978    0.000    0.978    0.000 copy.py:260(_keep_alive)
#   756271    0.709    0.000    0.709    0.000 ec.py:64(__lt__)
#    25800    0.700    0.000    0.700    0.000 random.py:264(sample)
#      430    0.431    0.001    1.985    0.005 selectors.py:208(tournament_selection)
#   491042    0.382    0.000    0.855    0.000 ec.py:76(__gt__)
#  1083027    0.293    0.000    0.293    0.000 copy.py:197(_deepcopy_atomic)
#        1    0.257    0.257   14.810   14.810 ec.py:167(evolve)
#    25860    0.091    0.000    0.167    0.000 ec.py:45(__init__)
# 155160/129300    0.087    0.000    0.087    0.000 ec.py:51(__setattr__)
#      430    0.079    0.000    0.182    0.000 replacers.py:223(comma_replacement)
#      431    0.073    0.000    0.134    0.000 archivers.py:48(best_archiver)
#    27186    0.073    0.000    0.073    0.000 random.py:516(gauss)
#    36167    0.027    0.000    0.027    0.000 ec.py:82(__eq__)
#      430    0.025    0.000    0.025    0.000 random.py:250(shuffle)
#     6360    0.015    0.000    0.015    0.000 random.py:147(randrange)
#===============================================================================
        
    



"""
    This module provides analysis methods for the results of evolutionary computations.

    .. Copyright (C) 2010  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import math
import csv


def generation_plot(filename, errorbars=True):
    """Plot the results of the EC using generation statistics.
    
    This function creates a plot of the generation fitness statistics 
    (best, worst, median, and average). 

    .. figure:: http://mcis.jsu.edu/faculty/agarrett/generation_plot.png
        :alt: Example generation plot
        :align: center
        
        An example image saved from the generation_plot function (without error bars).
    
    Arguments:
    
    - *filename* -- the name of the statistics file produced by the file_observer 
    - *errorbars* -- Boolean value stating whether standard error bars should 
      be drawn (default True)

    """
    import pylab
    import matplotlib.font_manager 
    
    generation = []
    psize = []
    worst = []
    best = []
    median = []
    average = []
    stdev = []
    reader = csv.reader(open(filename))
    for row in reader:
        generation.append(int(row[0]))
        psize.append(int(row[1]))
        worst.append(float(row[2]))
        best.append(float(row[3]))
        median.append(float(row[4]))
        average.append(float(row[5]))
        stdev.append(float(row[6]))
    stderr = [s / math.sqrt(p) for s, p in zip(stdev, psize)]
    
    data = [average, median, best, worst]
    colors = ['black', 'blue', 'green', 'red']
    labels = ['average', 'median', 'best', 'worst']
    figure = pylab.figure()
    if errorbars:
        pylab.errorbar(generation, average, stderr, color=colors[0], label=labels[0])
    else:
        pylab.plot(generation, average, color=colors[0], label=labels[0])
    for d, col, lab in zip(data[1:], colors[1:], labels[1:]):
        pylab.plot(generation, d, color=col, label=lab)
    pylab.fill_between(generation, data[2], data[3], color='#e6f2e6')
    pylab.grid(True)
    ymin = min([min(d) for d in data])
    ymax = max([max(d) for d in data])
    yrange = ymax - ymin
    pylab.ylim((ymin - 0.1*yrange, ymax + 0.1*yrange))  
    prop = matplotlib.font_manager.FontProperties(size=8) 
    pylab.legend(loc='upper left', prop=prop)    
    pylab.xlabel('Generation')
    pylab.ylabel('Fitness')
    pylab.show()    
    
    
def hypervolume(pareto_set, reference_point=None):
    """Calculates the hypervolume by slicing objectives (HSO).
    
    This function calculates the hypervolume (or S-measure) of a nondominated
    set using the Hypervolume by Slicing Objectives (HSO) procedure of `While, et al. 
    (IEEE CEC 2005) <http://www.lania.mx/~ccoello/EMOO/while05a.pdf.gz>`_.
    The *pareto_set* should be a list of lists of objective values.
    The *reference_point* may be specified or it may be left as the default 
    value of None. In that case, the reference point is calculated to be the
    maximum value in the set for all objectives (the ideal point). This function 
    assumes that objectives are to be maximized.
    
    Arguments:
    
    - *pareto_set* -- the list or lists of objective values comprising the Pareto front
    - *reference_point* -- the reference point to be used (default None)
    
    """
    def dominates(p, q, k=None):
        if k is None:
            k = len(p)
        d = True
        while d and k < len(p):
            d = not (q[k] > p[k])
            k += 1
        return d
        
    def insert(p, k, pl):
        ql = []
        while pl and pl[0][k] > p[k]:
            ql.append(pl[0])
            pl = pl[1:]
        ql.append(p)
        while pl:
            if not dominates(p, pl[0], k):
                ql.append(pl[0])
            pl = pl[1:]
        return ql

    def slice(pl, k, ref):
        p = pl[0]
        pl = pl[1:]
        ql = []
        s = []
        while pl:
            ql = insert(p, k + 1, ql)
            p_prime = pl[0]
            s.append((math.fabs(p[k] - p_prime[k]), ql))
            p = p_prime
            pl = pl[1:]
        ql = insert(p, k + 1, ql)
        s.append((math.fabs(p[k] - ref[k]), ql))
        return s

    ps = pareto_set
    ref = reference_point
    n = min([len(p) for p in ps])
    if ref is None:
        ref = [max(ps, key=lambda x: x[o])[o] for o in range(n)]
    pl = ps[:]
    pl.sort(key=lambda x: x[0], reverse=True)
    s = [(1, pl)]
    for k in range(n - 1):
        s_prime = []
        for x, ql in s:
            for x_prime, ql_prime in slice(ql, k, ref):
                s_prime.append((x * x_prime, ql_prime))
        s = s_prime
    vol = 0
    for x, ql in s:
        vol = vol + x * math.fabs(ql[0][n - 1] - ref[n - 1])
    return vol



    
    


__all__ = ['constraint_example', 'custom_ec_example', 'dea_example', 'eda_example', 'es_example', 'ga_example',
           'infinite_monkey_theorem', 'niche_example', 'nsga_example', 'paes_example', 'pso_example', 'sa_example']


from random import Random
from time import time
from ecspy import ec
from ecspy import terminators
from ecspy import selectors
from ecspy import replacers
from ecspy import variators
from ecspy import observers
from ecspy import benchmarks


def main(do_plot=True, prng=None):
    if prng is None:
        prng = Random()
        prng.seed(time()) 
    
    problem = benchmarks.Ackley(2)
    ea = ec.EvolutionaryComputation(prng)
    ea.selector = selectors.tournament_selection
    ea.variator = [variators.uniform_crossover, variators.gaussian_mutation]
    ea.replacer = replacers.steady_state_replacement
    ea.terminator = terminators.generation_termination
    final_pop = ea.evolve(generator=problem.generator,
                          evaluator=problem.evaluator,
                          pop_size=100, 
                          bounder=problem.bounder,
                          maximize=problem.maximize,
                          tourn_size=7,
                          num_selected=2, 
                          max_generations=300,
                          mutation_rate=0.2)
                          
    if do_plot:
        best = max(final_pop) 
        print('%s Example (%s) Best Solution: \n%s' % (ea.__class__.__name__, problem.__class__.__name__, str(best)))
        import itertools
        import pylab
        import mpl_toolkits.mplot3d.axes3d as axes3d
        num_points = 60
        points = []
        for lb, ub in zip(problem.bounder.lower_bound, problem.bounder.upper_bound):
            points.append([(i / float(num_points)) * (ub - lb) + lb for i in range(num_points)])
        points = itertools.product(*points)
        x = []
        y = []
        for p in points:
            x.append(p[0])
            y.append(p[1])
        z = problem.evaluator([[a, b] for a, b in zip(x, y)], {})
        fig = pylab.figure()
        ax = axes3d.Axes3D(fig)
        ax.scatter3D(x, y, z)
        ax.scatter3D([best.candidate[0]], [best.candidate[1]], [best.fitness], color='r')
        ax.scatter3D([problem.global_optimum[0]], [problem.global_optimum[1]], problem.evaluator([problem.global_optimum], {}), color='g')
        ax.set_xlabel('X')
        ax.set_ylabel('Y')
        ax.set_zlabel('Fitness')
        pylab.savefig('%s Example (%s).pdf' % (ea.__class__.__name__, problem.__class__.__name__), format='pdf')
        pylab.show()
    return ea

if __name__ == '__main__':
    main()



"""
    This module provides pre-defined observers for evolutionary computations.
    
    All terminator functions have the following arguments:
    
    - *population* -- the population of Individuals
    - *num_generations* -- the number of elapsed generations
    - *num_evaluations* -- the number of candidate solution evaluations
    - *args* -- a dictionary of keyword arguments    
    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import time
import math


def default_observer(population, num_generations, num_evaluations, args):
    """Do nothing."""    
    pass
    
    
def screen_observer(population, num_generations, num_evaluations, args):
    """Print the output of the EC to the screen.
    
    This function displays the results of the evolutionary computation
    to the screen. The output includes the generation number, the current
    number of evaluations, the average fitness, the maximum fitness, and 
    the full population.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    """
    import numpy
    
    population = list(population)
    population.sort(reverse=True)
    worst_fit = population[-1].fitness
    best_fit = population[0].fitness
    med_fit = numpy.median([p.fitness for p in population])
    avg_fit = numpy.mean([p.fitness for p in population])
    std_fit = numpy.std([p.fitness for p in population], ddof=1)

    print('Generation Evaluation Worst      Best       Median     Average    Std Dev   ')
    print('---------- ---------- ---------- ---------- ---------- ---------- ----------')
    print('{0:10} {1:10} {2:10} {3:10} {4:10} {5:10} {6:10}\n'.format(num_generations, num_evaluations, worst_fit, best_fit, med_fit, avg_fit, std_fit))
    print('Current Population:')
    for ind in population:
        print(str(ind))
    print('----------------------------------------------------------------------------')

    
def file_observer(population, num_generations, num_evaluations, args):
    """Print the output of the EC to a file.
    
    This function saves the results of the evolutionary computation
    to two files. The first file, which by default is named 
    'ecspy-statistics-file-<timestamp>.csv', contains the basic
    generational statistics of the population throughout the run
    (worst, best, median, and average fitness and standard deviation
    of the fitness values). The second file, which by default is named
    'ecspy-individuals-file-<timestamp>.csv', contains every individual
    during each generation of the run. Both files may be passed to the
    function as keyword arguments (see below).
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments

    Optional keyword arguments in args:
    
    - *statistics_file* -- a file object (default: see text)
    - *individuals_file* -- a file object (default: see text) 
    
    """
    # Import the necessary libraries here. Otherwise, they would have to be
    # installed even if this function is not called.
    import numpy
    
    try:
        statistics_file = args['statistics_file']
    except KeyError:
        statistics_file = open('ecspy-statistics-file-' + time.strftime('%m%d%Y-%H%M%S') + '.csv', 'w')
    try:
        individuals_file = args['individuals_file']
    except KeyError:
        individuals_file = open('ecspy-individuals-file-' + time.strftime('%m%d%Y-%H%M%S') + '.csv', 'w')

    population = list(population)
    population.sort(reverse=True)
    worst_fit = population[-1].fitness
    best_fit = population[0].fitness
    med_fit = numpy.median([p.fitness for p in population])
    avg_fit = numpy.mean([p.fitness for p in population])
    std_fit = numpy.std([p.fitness for p in population], ddof=1)
    statistics_file.write('{0}, {1}, {2}, {3}, {4}, {5}, {6}\n'.format(num_generations, len(population), worst_fit, best_fit, med_fit, avg_fit, std_fit))
    for i, p in enumerate(population):
        individuals_file.write('{0}, {1}, {2}, {3}\n'.format(num_generations, i, p.fitness, str(p.candidate)))
    statistics_file.flush()
    individuals_file.flush()
    

def archive_observer(population, num_generations, num_evaluations, args):
    """Print the current archive to the screen."""
    archive = args['_ec'].archive
    print('                         Archive Size: %5d' % len(archive))
    print('----------------------------------------------------------------------')
    for a in archive:
        print(a)
    print('----------------------------------------------------------------------')

        
def plot_observer(population, num_generations, num_evaluations, args):    
    """Plot the output of the EC as a graph.
    
    This function plots the performance of the EC as a line graph 
    using the pylab library (matplotlib). The graph consists of a 
    blue line representing the best fitness, a green line representing
    the average fitness, and a red line representing the median fitness.
    It modifies the keyword arguments variable 'args' by including an
    entry called 'plot_data'.
    
    If this observer is used, the calling script should also import
    the pylab library and should end the script with 
    
    pylab.show()
    
    Otherwise, the program may generate a runtime error.
    
    .. Arguments:
       population -- the population of Individuals
       num_generations -- the number of elapsed generations
       num_evaluations -- the number of candidate solution evaluations
       args -- a dictionary of keyword arguments
    
    """
    import pylab
    import numpy
    
    population = list(population)
    population.sort(reverse=True)
    best_fitness = population[0].fitness
    worst_fitness = population[-1].fitness
    median_fitness = numpy.median([p.fitness for p in population])
    average_fitness = numpy.mean([p.fitness for p in population])
    colors = ['black', 'blue', 'green', 'red']
    labels = ['average', 'median', 'best', 'worst']
    data = []
    if num_generations == 0:
        pylab.ion()
        data = [[num_evaluations], [average_fitness], [median_fitness], [best_fitness], [worst_fitness]]
        lines = []
        for i in range(4):
            line, = pylab.plot(data[0], data[i+1], color=colors[i], label=labels[i])
            lines.append(line)
        # Add the legend when the first data is added.
        pylab.legend(loc='lower right')
        args['plot_data'] = data
        args['plot_lines'] = lines
        pylab.xlabel('Evaluations')
        pylab.ylabel('Fitness')
    else:
        data = args['plot_data']
        data[0].append(num_evaluations)
        data[1].append(average_fitness)
        data[2].append(median_fitness)
        data[3].append(best_fitness)
        data[4].append(worst_fitness)
        lines = args['plot_lines']
        for i, line in enumerate(lines):
            line.set_xdata(numpy.array(data[0]))
            line.set_ydata(numpy.array(data[i+1]))
        args['plot_data'] = data
        args['plot_lines'] = lines
    ymin = min([min(d) for d in data[1:]])
    ymax = max([max(d) for d in data[1:]])
    yrange = ymax - ymin
    pylab.xlim((0, num_evaluations))
    pylab.ylim((ymin - 0.1*yrange, ymax + 0.1*yrange))
    pylab.draw()


"""
    Evaluator functions are problem-specific. This module provides pre-defined 
    evaluators for evolutionary computations.

    All evaluator functions have the following arguments:
    
    - *candidates* -- the candidate solutions
    - *args* -- a dictionary of keyword arguments

    
    .. Copyright (C) 2009  Inspired Intelligence Initiative

    .. This program is free software: you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation, either version 3 of the License, or
       (at your option) any later version.

    .. This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

    .. You should have received a copy of the GNU General Public License
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def evaluator(evaluate):
    """Return an ecspy evaluator function based on the given function.
    
    This function generator takes a function that evaluates only one
    candidate. The generator handles the iteration over each candidate 
    to be evaluated.

    The given function ``evaluate`` must have the following signature::
    
        fitness = evaluate(candidate, args)
        
    This function is most commonly used as a function decorator with
    the following usage::
    
        @evaluator
        def evaluate(candidate, args):
            # Implementation of evaluation
            
    The generated function also contains an attribute named
    ``single_evaluation`` which holds the original evaluation function.
    In this way, the original single-candidate function can be
    retrieved if necessary.
    
    """
    def ecspy_evaluator(candidates, args):
        fitness = []
        for candidate in candidates:
            fitness.append(evaluate(candidate, args))
        return fitness
    ecspy_evaluator.__dict__ = evaluate.__dict__
    ecspy_evaluator.single_evaluation = evaluate
    ecspy_evaluator.__name__ = evaluate.__name__
    ecspy_evaluator.__doc__ = evaluate.__doc__
    return ecspy_evaluator
    

def parallel_evaluation_pp(candidates, args):
    """Evaluate the candidates in parallel using Parallel Python.

    This function allows parallel evaluation of candidate solutions.
    It uses the Parallel Python (pp) library to accomplish the 
    parallelization. This library must already be installed in order
    to use this function. The function assigns the evaluation of each
    candidate to its own job, all of which are then distributed to the
    available processing units.
    
    parallel_evaluation_mp is the slightly better choice for SMP/multicore 
    parallelism since it does not require you to specify arguments and 
    modules required for evaluation in a non-standard manner. It is, 
    therefore, trivial to set up.
    
    parallel_evaluation_pp supports both SMP / multicore parallelism,
    as well as distributed computing, which does require you to set up 
    a network of clients. 
    
    .. Arguments:
       candidates -- the candidate solutions
       args -- a dictionary of keyword arguments

    Required keyword arguments in args:
    
    *pp_evaluator* -- actual evaluation function to be used (This function
    should have the same signature as any other ecspy evaluation function.)

    Optional keyword arguments in args:
    
    - *pp_dependencies* -- tuple of functional dependencies of the serial 
      evaluator (default ())
    - *pp_modules* -- tuple of modules that must be imported for the 
      functional dependencies (default ())
    - *pp_servers* -- tuple of servers (on a cluster) that will be used 
      for parallel processing (default ("*",))
      
    For more information about these arguments, please consult the
    documentation for Parallel Python.
    
    """
    try:
        import pp
    except ImportError:
        print '''Parallel Python is not installed...\n
        The parallel_evaluation_pp function requires Parallel Python.\n'''
        raise
    logger = args['_ec'].logger
    
    try:
        evaluator = args['pp_evaluator']
    except KeyError:
        logger.error('\'mp_evaluator\' is not in the keyword arguments list')
        raise 
    try:
        job_server = args['_pp_job_server']
    except KeyError:
        pp_servers = args.get('pp_servers', ("*",))
        job_server = pp.Server(ppservers=pp_servers)
        args['_pp_job_server'] = job_server
    pp_depends = args.setdefault('pp_dependencies', ())
    pp_modules = args.setdefault('pp_modules', ())
        
    func_template = pp.Template(job_server, evaluator, pp_depends, pp_modules)
    jobs = [func_template.submit([c], {}) for c in candidates]
    
    results = []
    for job in jobs:
        r = job()
        results.append(r[0])
    
    fitness = []
    for result in results:
        fitness.append(result)
    return fitness

def parallel_evaluation_mp(candidates, args):
    """Evaluate the candidates in parallel using ``multiprocessing``.

    This function allows parallel evaluation of candidate solutions.
    It uses the standard multiprocessing library to accomplish the 
    parallelization. The function assigns the evaluation of each
    candidate to its own job, all of which are then distributed to the
    available processing units.
    
    parallel_evaluation_mp is the slightly better choice for SMP/multicore 
    parallelism since it does not require you to specify arguments and 
    modules required for evaluation in a non-standard manner. It is, 
    therefore, trivial to set up.
    
    parallel_evaluation_pp supports both SMP / multicore parallelism,
    as well as distributed computing, which does require you to set up 
    a network of clients. 
    
    Note: arguments for the evaluation function should be able to serialize
    
    .. Arguments:
       candidates -- the candidate solutions
       args -- a dictionary of keyword arguments

    Required keyword arguments in args:
    
    *mp_evaluator* -- actual evaluation function to be used (This function
    should have the same signature as any other ecspy evaluation function.)

    Optional keyword arguments in args:
    
    - *mp_num_cpus* -- number of processors that will be used (default is machine cpu count)
    
    """
    import time
    try:
        import multiprocessing
    except ImportError:
        print '''multiprocessing is not installed...\n
        ecspy has been designed to work with Python 2.6, which has multiprocessing as a standard library\n'''
        raise
    logger = args['_ec'].logger
    
    try:
        evaluator = args['mp_evaluator']
    except KeyError:
        logger.error('\'mp_evaluator\' is not in the keyword arguments list')
        raise 
    try:
        nprocs = args['mp_num_cpus']
    except KeyError:
        nprocs = multiprocessing.cpu_count()
    
    start = time.time()
    try:
        pool = multiprocessing.Pool(processes=nprocs)
        results = [pool.apply_async(evaluator, ([c], {})) for c in candidates]
        pool.close()
        return [r.get()[0] for r in results]
    except (OSError, RuntimeError) as e:
        logger.error('failed parallel fitness evaluation using multiprocessing')
        raise
    else:
        end = time.time()
        logger.debug('completed parallel evaluation in %f seconds' % (end - start))
        


'''
Created on Dec 12, 2010

@author: Henrik Rudstrom
'''
import ecspy.contrib.smcgp.modification_functions as mod
from ecspy.contrib.smcgp.encoding import SMCGPGene, SMCGPEncoding, call_mod
import unittest
from ecspy.contrib.smcgp.phenotype import io_functions, SMCGPPhenotype

class SMCGPDummyEncoding(SMCGPEncoding):
    def random_gene(self, col):
        return SMCGPGene(3,[1,1],[0,0,0])

def input_gene():
    return SMCGPGene(0,[0,0], [0,0,0])    
def output_gene(inp):
    return SMCGPGene(2,[inp,inp],[0,0,0])
def func_gene(f, a):
    return SMCGPGene(f, a,[0,0,0])

def mod_gene(f, inp, vars):
    f = mod.all_functions.index(f)+5
    return SMCGPGene(f, inp, vars)


class ModTest(unittest.TestCase):
    def setUp(self):
        def add(gene, a, b):
            return a + b
        def sub(gene, a, b):
            return a - b
        
        self.enc = SMCGPDummyEncoding(2, 2, 1, 1)
        
        self.user = [add, sub]
        self.enc.set_functions(io_functions+self.user+mod.all_functions)
    
    def test_add(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.add,[2,1],[1,2,0]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))
        modified = func.get_modified()
        assert len(func.candidate) == 6
        assert len(modified.candidate) == 8
        assert func.candidate[:3] == modified.candidate[:3]
        assert func.candidate[3:] == modified.candidate[5:]
        #print "mod", modified.candidate
        
        mod2 = modified.get_modified()
        #print "mod2", mod2.candidate
        #print "mod2",len(mod2.candidate)
        assert len(mod2.candidate) == 10
        assert modified.candidate[:3] == mod2.candidate[:3]
        assert modified.candidate[3:] == mod2.candidate[5:]
        
    def test_del(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.delete,[2,1],[1,2,0]), func_gene(3,[1,1]), func_gene(4,[2,1]),func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))
        modified = func.get_modified()
        assert len(func.candidate) == 7
        assert len(modified.candidate) == 5
        assert func.candidate[:3] == modified.candidate[:3]
        assert func.candidate[5:] == modified.candidate[3:] 
        mod2 = modified.get_modified()  
        print func.candidate
        print modified.candidate
        print mod2.candidate
        assert len(mod2.candidate) == 3; #no more to delete
        assert modified.candidate[:3] == mod2.candidate[:3]
        #assert modified.candidate[5:] == mod2.candidate[3:]
    def test_del_add(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.add,[2,1],[3,3,0]), mod_gene(mod.delete,[2,1],[1,1,0]), func_gene(3,[1,2]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))
        modified = func.get_modified()
        assert len(func.candidate) == 7
        #print len(modified.candidate)
        assert len(modified.candidate) == 9
        deleted = gen1[4]
        #print deleted
        #print modified.candidate
        assert deleted not in modified.candidate
        assert func.candidate[:3] == modified.candidate[:3]
        assert func.candidate[5:] == modified.candidate[7:]
        
    def test_dup(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.duplicate,[2,1],[1,2,1]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        assert len(modified.candidate) == len(func.candidate) + 2
        assert func.candidate[3:4] == modified.candidate[3:4]
        assert func.candidate[3:4] == modified.candidate[5:6]
        
    def test_overwrite(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.overwrite,[2,1],[1,1,2]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        assert len(modified.candidate) == len(func.candidate)
        assert func.candidate[3] == modified.candidate[3]
        assert func.candidate[4] == modified.candidate[4]
        
    def test_dup_preserve(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.duplicate_and_shift,[2,1],[1,2,3]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        assert len(modified.candidate) == len(func.candidate) + 2
        assert func.candidate[3:4] == modified.candidate[3:4]
        assert func.candidate[3].function == modified.candidate[5].function
        assert func.candidate[3].vars[0] == modified.candidate[5].vars[0]
        assert func.candidate[3].vars[1] == modified.candidate[5].vars[1]
        assert func.candidate[3].inputs[0] + 2 == modified.candidate[5].inputs[0]
        assert func.candidate[3].inputs[1] + 2 == modified.candidate[5].inputs[1]
        assert func.candidate[4].function == modified.candidate[6].function
        assert func.candidate[4].vars[0] == modified.candidate[6].vars[0]
        assert func.candidate[4].vars[1] == modified.candidate[6].vars[1]
        assert func.candidate[4].inputs[0] + 2 == modified.candidate[6].inputs[0]
        assert func.candidate[4].inputs[1] + 2 == modified.candidate[6].inputs[1]
        
    def test_dup_scale(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.duplicate_and_scale,[2,1],[1,2,3]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        assert len(modified.candidate) == len(func.candidate) + 2
        assert func.candidate[3:4] == modified.candidate[3:4]
        assert func.candidate[3].function == modified.candidate[5].function
        assert func.candidate[3].vars[0] == modified.candidate[5].vars[0]
        assert func.candidate[3].vars[1] == modified.candidate[5].vars[1]
        assert func.candidate[3].inputs[0] * 3 == modified.candidate[5].inputs[0]
        assert func.candidate[3].inputs[1] * 3 == modified.candidate[5].inputs[1]
        assert func.candidate[4].function == modified.candidate[6].function
        assert func.candidate[4].vars[0] == modified.candidate[6].vars[0]
        assert func.candidate[4].vars[1] == modified.candidate[6].vars[1]
        assert func.candidate[4].inputs[0] * 3 == modified.candidate[6].inputs[0]
        assert func.candidate[4].inputs[1] * 3 == modified.candidate[6].inputs[1]
     
    
    def test_shift_connections(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.shift_connections,[2,1],[1,2,3]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        
        assert len(modified.candidate) == len(func.candidate)
        assert func.candidate[3].inputs[0] + 3 == modified.candidate[3].inputs[0]
        assert func.candidate[3].inputs[1] + 3 == modified.candidate[3].inputs[1]
    
    def test_scale_connections(self):
        gen1 = [input_gene(), input_gene(), mod_gene(mod.scale_connections,[2,1],[1,2,3]), func_gene(3,[1,1]), func_gene(4,[2,1]), output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        
        assert len(modified.candidate) == len(func.candidate)
        assert func.candidate[3].inputs[0] * 3 == modified.candidate[3].inputs[0]
        assert func.candidate[3].inputs[1] * 3 == modified.candidate[3].inputs[1] 
        
    def test_change_functions(self):
        gen1 = [input_gene(), input_gene(),func_gene(3,[1,1]),func_gene(4,[1,1]),
                mod_gene(mod.change_connection,[2,1],[2,0,2]),
                mod_gene(mod.change_function,[2,1],[2,3,0]),
                mod_gene(mod.change_parameter,[2,1],[2,0,3]),  output_gene(1)]
        func = SMCGPPhenotype(self.enc, list(gen1))   
        modified = func.get_modified()
        assert modified.candidate[2].function == modified.candidate[3].function
        
        assert modified.candidate[2].inputs[0] == 3
        assert modified.candidate[2].vars[0] == 3
        assert len(modified.candidate) == len(func.candidate)
        
    def test_input_output(self):
        enc = SMCGPEncoding(2, 1, 1, 2)
        enc.set_functions(io_functions)
        gen = [input_gene(), input_gene(), output_gene(2), output_gene(2), output_gene(3)]
        
        #gen = list_to_genes([[0,0],[0,0], [2, 2], [2,2], [2,3]])
        input = [5,2]
        func = SMCGPPhenotype(enc, gen)
        #print "funcs", func.functions
        res = func(input)
        assert res[0] == input[0]
        assert res[1] == input[1]
        assert res[2] == input[1]
        
    def test_no_outputs(self):
        enc = SMCGPEncoding(2, 1, 1, 2)
        enc.set_functions(io_functions)
        gen = [input_gene(), input_gene()]
        input = [5,2]
        func = SMCGPPhenotype(enc, gen)  

        res = func(input)
        
        assert res[0] == input[0], "res"+res[0]
        assert res[1] == input[1], "res"+res[1]

    def test_too_few_outputs(self):
        enc = SMCGPEncoding(2, 1, 1, 2)
        enc.set_functions(io_functions)
        gen = [input_gene(), input_gene(), output_gene(2)]
        input = [5,2]
        func = SMCGPPhenotype(enc, gen)    
        res = func(input)
        
        assert res[0] == input[0]
        assert res[1] == input[1]  
        
    def test_no_inputs(self):
        enc = SMCGPEncoding(2, 1, 1, 2)
        enc.set_functions(io_functions)
        gen = [output_gene(2), output_gene(2)]
        func = SMCGPPhenotype(enc, gen)
        assert func([1,2]) == [1,1] #first node becomes input
        
        
    def test_arithmetics(self):
        def add(gene, a, b):
            
            return a + b
        def sub(gene, a, b):
            return a - b
        enc = SMCGPEncoding(2, 2, 1, 1)
        
        enc.set_functions(io_functions+[add, sub])
        gen1 = [input_gene(),input_gene(), func_gene(3,[2,1]), func_gene(4,[2,1]), output_gene(1)]
        
        #gen1 = list_to_genes([[0,0,0],[0,0,0], [3,2,1], [3,2,2], [4, 2, 1], [2,1,0]])
        func = SMCGPPhenotype(enc, gen1) 
        input = [1,10]
        #(1 + 10) - (10 + 10) 
        res = func(input)
        #print "RES", res
        assert res[0] == -1
        
        
    def test_bias(self):
        
        count = [0,0,0]
        ranges = [self.enc.io_func_range, self.enc.user_func_range, self.enc.mod_func_range]
        
        for _ in xrange(1000):
            f = self.enc.random_func(0)
            for i,(s,e) in enumerate(ranges):
                if f >= s and f < e:
                    count[i] += 1
                    break

        def almost_equal(c1, c2):
            print c1, c2
            return abs(c1-c2) < 50
        for a in count:
            for b in count:
                assert almost_equal(a,b)
            
            

if __name__ == '__main__':
    unittest.main()


'''
Created on Dec 13, 2010

@author: Henrik Rudstrom



`we evolve for two input bits. When a successful solution is found,
the fitness function requires that the program produces a two bit
circuit, followed by a three bit circuit. Then when a genotype has
been found that solves the two bit problem and on iteration solves
the three bit problem, the fitness function changes so that now in
addition to the previous behaviour the genotype, when iterated twice,
produces a phenotype that solves the four-bit problem. The process
continues in this way until we obtain a phenotype that correctly
implements the required function with 20 inputs. We refer to the
application of each parity or adder as a test case.
Fitness is computed as the number of correctly predicted bits over all
test cases. If the candidate solution fails to find a totally correct
solution for a given input size, it is not tested on other input sizes.
We evolve for 19 test cases (2 inputs to 20 inputs).`
-- "Developments in Cartesian Genetic Programming: self-modifying CGP",
   Harding Miller Banzhaf 2010


'''

import random
from random import Random
import time
#import pyximport;
#pyximport.install(pyximport, pyimport, build_dir, build_in_temp, setup_args, reload_support)
from ecspy import ec, terminators
from ecspy.contrib.smcgp.encoding import SMCGPEncoding
from ecspy.contrib.smcgp.phenotype import io_functions, SMCGPPhenotype
from ecspy.contrib.smcgp import modification_functions

def wrap(name, func):
    def f(g,a,b):
        return func(g,a,b)
    f.__name__ = name
    return f

user_functions = [
    wrap("BAND", lambda g,a,b: a and b), 
    wrap("BOR", lambda g,a,b: a or b),
    wrap("BNAND", lambda g,a,b: not  (a and b)),
    wrap("BXOR",  lambda g,a,b: (a and not  b) or (b and not  a)), 
    wrap("BNOR",  lambda g,a,b: not  (a or b)),
    wrap("BNOT", lambda g,a,b: not a),
    wrap("BIAND", lambda g,a,b: (not  a) and b), 
    wrap("BF0", lambda g,a,b: False),
    wrap("BF1", lambda g,a,b: (a and b)), 
    wrap("BF2", lambda g,a,b: a and (not  b)), 
    wrap("BF3", lambda g,a,b: (a and (not b))or(a and b)), 
    wrap("BF4", lambda g,a,b: (not  a) and b),
    wrap("BF5", lambda g,a,b: ((not a) and b) or (a and b)), 
    wrap("BF6", lambda g,a,b: ((not  a) and b) or (a and (not  b))), 
    wrap("BF7", lambda g,a,b: ((not  a) and b) or (a and not (b)) or (a and b)), 
    wrap("BF8", lambda g,a,b: ((not  a) and (not  b))),
    wrap("BF9", lambda g,a,b: ((not  a) and (not  b)) or (a and b)), 
    wrap("BF10", lambda g,a,b: ((not  a) and (not  b)) or (a and not  (b))), 
    wrap("BF11", lambda g,a,b: ((not a) and (not b) or a and (not b) or a and b)), 
    wrap("BF12", lambda g,a,b: ((not  a) and (not  b) or (not  a) and b)),
    wrap("BF13", lambda g,a,b: ((not a) and (not b) or (not a) and b or a and b)), 
    wrap("BF14", lambda g,a,b: ((not a) and (not b) or (not a) and b or a and (not b))), 
    wrap("BF15", lambda g,a,b: ((not a) and (not b) or (not a) and b or a and (not b) or a and b))
]

enc = SMCGPEncoding(length=25, max_arity=2, levelsback=15, min_outputs=1)
enc.set_functions(io_functions+user_functions+modification_functions.all_functions)


def int2bin(n, count=24):
    """returns the binary of integer n, using count number of digits"""
    return [(n >> y) & 1 for y in range(count-1, -1, -1)]

def get_test_cases(max_bits):
    return [[int2bin(i, bits) for i in xrange(2**bits)] for bits in xrange(2,max_bits+2)]

def get_targets(all_cases):
    return [[not sum(case) % 2 for case in cases] for cases in all_cases]

def select_test_cases(all_cases, all_targets, max_cases=100):
    cases = []
    targets = []
    for case, target in zip(all_cases, all_targets):
        if len(case) < max_cases:
            cases.append(case)
            targets.append(target)
        else:
            sel = list(xrange(len(case)))
            random.shuffle(sel)
            cases.append([case[s] for s in sel[:max_cases]])
            targets.append([target[s] for s in sel[:max_cases]])
    return cases, targets   


def test_test_cases():
    ac = get_test_cases(10)
    tac = get_targets(ac)
    lac = [len(c) for c in ac]
    print lac
    assert lac == [4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048]
    sac,stac = select_test_cases(ac, tac, 100)
    lsac = [len(c) for c in sac]
    print lsac
    assert lsac == [4, 8, 16, 32, 64, 100, 100, 100, 100, 100]
    for c,t in zip(sac, stac):
        assert len(c) % 2 == 0

test_test_cases()

all_cases = get_test_cases(10)
all_targets = get_targets(all_cases)
def fitness(candidates, args):
    sel_cases, sel_targets = select_test_cases(all_cases, all_targets, 100)
    fitness = []
    for cand in candidates:
        fit = 0
        func = SMCGPPhenotype(enc, cand)
        iii=0
        for cases, targets in zip(sel_cases, sel_targets):
            
            iii += 1
            func = func.get_modified()
            if not func.is_valid():
                break
            case_fit = 0
            for case, target in zip(cases, targets):
                solution = func(case)[0]
                if solution == target:
                    #print solution, target, case
                    case_fit += 1
            fit += case_fit
            if case_fit != len(cases):
                break
        #print "cases", iii
        fitness.append(fit)
    return fitness

            
            
def screen_observer(enc):
    def screen_observer(population, num_generations, num_evaluations, args):
        import numpy
        
        population = list(population)
        population.sort(reverse=True)
        worst_fit = population[-1].fitness
        best_fit = population[0].fitness
        med_fit = numpy.median([p.fitness for p in population])
        avg_fit = numpy.mean([p.fitness for p in population])
        std_fit = numpy.std([p.fitness for p in population], ddof=1)
        if(num_generations % 50 != 0):
            return
        print('Generation Evaluation Worst      Best       Median     Average    Std Dev   ')
        print('---------- ---------- ---------- ---------- ---------- ---------- ----------')
        print('{0:10} {1:10} {2:10} {3:10} {4:10} {5:10} {6:10}\n'.format(num_generations, num_evaluations, worst_fit, best_fit, med_fit, avg_fit, std_fit))
        print('Current Population: (id,fitness,todo,2 bit solution)')
        print('----------------------------------------------------------------------------')
        for i, c in enumerate(population):
            #f = CGPPhenotype(enc, c.candidate)
            #exp = f([Symbol('x'), RealNumber(1.0)])
            pass
        for i, c in enumerate(population):
            #f = CGPPhenotype(enc, c.candidate)
            #exp = f([Symbol('x'), RealNumber(1.0)])
            phe = SMCGPPhenotype(enc, c.candidate)
            #print i, c.fitness, phe.todo_list, phe
            print i, c.fitness, [p.__name__ for p in phe.todo_list], phe.get_modified()
            
    return screen_observer        


if __name__ == '__main__':
    
    mutator = enc.mutator()
    generator = enc.generator()
    #seeds = generate_initial_population(generator, 500, 4, fitness_function)

    prng = Random()
    prng.seed(time.time())
    ga = ec.ES(prng)

    ga.observer = [screen_observer(enc)]
    ga.terminator = terminators.evaluation_termination
    ga.variator = [mutator]
    #ga.selector = tournament_selection
    start = time.time()
    #print "setup", seeds


    final_pop = ga.evolve(#seeds = seeds,
                          evaluator=fitness,
                          generator=generator,
                          max_evaluations=100000000,
                          pop_size=5,
                          mutation_rate=0.066,
                          maximize=True,
                          use_one_fifth_rule=True)
    stop = time.time()

    print('***********************************')
    print('Total Execution Time: %0.5f seconds' % (stop - start))



